/*
 ============================================================================
 Name        : multi_card.c
 Author      : www.d-logic.net
 Version     : 1.0
 Description : anti-collision-demo console example written in C
 Description : For demonstration purposes works only with classic cards in
 Description :     transport configuration (tries to authenticate with key B
 Description :     value FF:FF:FF:FF:FF:FF).
 Dependency  : uFCoder library, min. ver. 4.0.19
 Dependency  : uFR firmware from ver. 4.0.6
 ============================================================================
 */

#define STR_APP_VERSION "1.0"

#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#if __WIN32 || __WIN64
#	include <conio.h>
#elif linux || __linux__ || __APPLE__
#	include "conio_gnu.h"
#endif
#include "uFCoder.h"
#include "anti_collision.h"
//------------------------------------------------------------------------------
int main(void) {
#define CONTENT_BUFF_LEN	2048
#define STR_BUFF_LEN(x)		STR_BUFF_LEN1(x)
#define STR_BUFF_LEN1(x)	#x
	UFR_STATUS status;
	int8_t IsAntiCollEnabled;
	int8_t IsAnyCardSelected;
	uint8_t CardsNumber;
	uint8_t List[4 * 11]; // Max list size is 44 bytes, first byte is always current UidSize
	uint8_t ListSize;
	uint8_t ListOfSizes[4];
	uint8_t ListOfIndexes[4];
	uint8_t index;
	uint8_t CardType;
	int loop = 1;
	int cancel = 0;
	char c;
	uint8_t key[6] = {0xFF,0xFF,0xFF,0xFF,0xFF,0xFF};
	uint8_t selected;
	int nitems, block;
	uint8_t data[17];
	uint8_t sak;
	uint8_t uid_len;

	printf( "\nConsole application example anti-collision-demo v" STR_APP_VERSION "\n"
			"Anti-collision-demo mode demonstration using advantages of the\n"
			"ISO-14443-3 anti collision algorithm.\n\n");
	printf( "uFCoder Library version: %s\n"
			"Please wait while trying connect to the uFR device.\n",
			GetDllVersionStr());

	status = ReaderOpen();
	if (status != UFR_OK) {
		printf("Can't connect to the device. Is uFR connected?\n");
		return EXIT_FAILURE;
	}

	usage();

	while (loop) {
		switch (getch()) {
		case 'I': // Get reader description:
		case 'i':
			printf("%s\n", GetReaderDescription());
			print_ln('=');
			break;
			//..................................................................
		case 'O': // Get software versions:
		case 'o':
			printf("anti-collision-demo console example version: " STR_APP_VERSION "\n");
			printf("FCoder Library version: %s\n", GetDllVersionStr());
			print_ln('=');
			break;
			//..................................................................
		case 'P': // Get ID of the tag in reader field:
		case 'p':
			status = GetCardIdEx(&sak, data, &uid_len);
			if (status != UFR_OK) {
				printf("Error, status is: 0x%08X\n", status);
				print_ln('=');
				break;
			}
			status = GetDlogicCardType(&CardType);
			if (status != UFR_OK) {
				printf("Error, status is: 0x%08X\n", status);
				print_ln('=');
				break;
			}
			printf("Card type: %s, uid[%d] = ", GetDlTypeName(CardType), uid_len);
			print_hex_ln(data, uid_len, ":");
			print_ln('=');
			break;
			//..................................................................
		case 'M': // Show current device mode:
		case 'm':
			status = GetAntiCollisionStatus(&IsAntiCollEnabled, &IsAnyCardSelected);
			if (!status)
				if (!IsAntiCollEnabled)
					printf("Anti-collision mode is disabled.\n");
				else
				{
					printf("Anti-collision mode is enabled.\n");
					if (IsAnyCardSelected)
						printf("One card is selected.\n");
					else
						printf("There are no card selected.\n");
				}
			else
				printf("Error, status is: 0x%08X\n", status);
			print_ln('=');
			break;
			//..................................................................
		case 'E': // Enable anti collision mode:
		case 'e':
			status = EnableAntiCollision();
			if (!status)
				printf("Anti-collision mode successfully enabled.\n");
			else
				printf("Error, status is: 0x%08X\n", status);
			print_ln('=');
			break;
			//..................................................................
		case 'D': // Disable anti collision mode:
		case 'd':
			status = DisableAntiCollision();
			if (!status)
				printf("Anti-collision mode successfully disabled.\n");
			else
				printf("Error, status is: 0x%08X\n", status);
			print_ln('=');
			CardsNumber = 0;
			break;
			//..................................................................
		case 'L': // List cards in reader field:
		case 'l':
			status = EnumCards(&CardsNumber, &ListSize);
			if (!status)
			{
				printf("There is %d cards in the reader field.\n", CardsNumber);
			}
			else
			{
				printf("Error, status is: 0x%08X\n", status);
				print_ln('=');
				break;
			}
			if (CardsNumber > 0) {
				status = ListCards(List, ListSize);
				if (!status)
				{
					printf("Uid list of the cards in the reader field are:\n");
					index = 0;
					for (int i = 0; i < CardsNumber; i++) {
						ListOfSizes[i] = List[index];
						ListOfIndexes[i] = index + 1;
						printf("%d. ", i + 1);
						print_hex_ln(&List[ListOfIndexes[i]], ListOfSizes[i], ":");
						index += 11;
					}
				}
				else
				{
					printf("Error, status is: 0x%08X\n", status);
					print_ln('.');
					print_hex_ln(List, ListSize, ":");
					print_ln('=');
					break;
				}
			}
			print_ln('=');
			break;
			//..................................................................
		case 'S': // Select listed card:
		case 's':
			if (!CardsNumber)
			{
				printf("Error: no card enumerated. Enumerate cards first.\n");
				print_ln('=');
				break;
			}
			printf("Choose card uid:\n");
			for (int i = 0; i < CardsNumber; i++) {
				printf("  [%d] ", i + 1);
				print_hex_ln(&List[ListOfIndexes[i]], ListOfSizes[i], ":");
			}
			printf("[esc] Cancel operation.\n");
			fflush(stdin);
			do {
				c = getch();
				switch (c) {
					case '1':
					case '2':
					case '3':
					case '4':
						selected = c - 0x30; // to int
						if (selected > CardsNumber)
							break;
						status = SelectCard(&List[ListOfIndexes[selected - 1]], ListOfSizes[selected - 1], &CardType);
						if (!status)
							printf("Card successfully selected. Selected card type is %s\n", GetDlTypeName(CardType));
						else
							printf("Error, status is: 0x%08X\n", status);
						cancel = 1;
						break;
					case '\x1b': // cancel:
						cancel = 1;
						break;
				}
			} while (!cancel);
			fflush(stdin);
			print_ln('=');
			break;
			//..................................................................
		case 'X': // Deselect card:
		case 'x':
			status = DeslectCard();
			if (!status)
				printf("Card successfully deselected.\n");
			else
				printf("Error, status is: 0x%08X\n", status);
			print_ln('=');
			break;
			//..................................................................
		case 'R': // Block Read:
		case 'r':
			printf("    Enter block address: ");
			nitems = scanf("%d", &block);
			fflush(stdin);
			if ((nitems == EOF) || (nitems == 0)) {
				printf("\nInput error.\n");
				print_ln('=');
				break;
			}
			if ((block < 0) || (block > 0xFF)) {
				printf("\nBlock out of range.\n");
				print_ln('=');
				break;
			}
			status = BlockRead_PK(data, block, MIFARE_AUTHENT1A, key);
			if (!status)
			{
				data[16] = 0;
				printf("Block successfully read. Content is [ASCII]: %s\n", data);
			}
			else
				printf("Error, status is: 0x%08X\n", status);
			print_ln('=');
			break;
			//..................................................................
		case 'W': // Block Write:
		case 'w':
			printf("    Enter block address: ");
			nitems = scanf("%d", &block);
			fflush(stdin);
			if ((nitems == EOF) || (nitems == 0)) {
				printf("\nInput error.\n");
				print_ln('=');
				break;
			}
			if ((block < 0) || (block > 0xFF)) {
				printf("\nPage out of range.\n");
				print_ln('=');
				break;
			}
			printf("    Enter block content [16 ASCII characters]: ");
			uint8_t *temp;
			temp = malloc(1024);
			temp[0] = 0;
			nitems = scanf("%s", temp);
			fflush(stdin);
			if ((nitems == EOF) || (nitems == 0) || strlen((char *)temp) != 16) {
				printf("\nInput error.\n");
				print_ln('=');
				free(temp);
				break;
			}
			status = BlockWrite_PK(temp, block, MIFARE_AUTHENT1B, key);
			if (!status)
				printf("Block successfully written.\n");
			else
				printf("Error, status is: 0x%08X\n", status);
			print_ln('=');
			free(temp);
			break;
			//..................................................................
		case 'H': // Show help screen:
		case 'h':
			usage();
			break;
			//..................................................................
		case '\x1b': // escape application:
			status = ReaderClose();
			if (status != UFR_OK) {
				printf("Error while closing device.\n");
				return EXIT_FAILURE;
			}
			loop = 0;
			break;
		}
	}

	return EXIT_SUCCESS;
}
//------------------------------------------------------------------------------
void usage(void) {
	print_ln('=');
	printf("\nCommand keys (case insensitive):\n");
	print_ln('-');

	printf( "  [I] Get reader description.\n"
			"  [O] Get software versions.\n"
			"  [P] Get ID of the selected tag.\n"

			"  [M] Show current device mode.\n"
			"  [E] Enable anti collision mode.\n"
			"  [D] Disable anti collision mode.\n"

			"  [L] List cards in reader field.\n"
			"  [S] Select listed card.\n"
			"  [X] Deselect card.\n"

			"  [R] Block Read.\n"
			"  [W] Block Write.\n"

			"  [H] This help screen.\n"
			"[esc] Close device and exit program.\n");
	print_ln('=');
}
//------------------------------------------------------------------------------
char *GetDlTypeName(uint8_t dl_type_code) {
	static char s[50];

	switch (dl_type_code) {
	case DL_MIFARE_ULTRALIGHT:
		strcpy(s, "DL_MIFARE_ULTRALIGHT");
		break;
	case DL_MIFARE_ULTRALIGHT_EV1_11:
		strcpy(s, "DL_MIFARE_ULTRALIGHT_EV1_11");
		break;
	case DL_MIFARE_ULTRALIGHT_EV1_21:
		strcpy(s, "DL_MIFARE_ULTRALIGHT_EV1_21");
		break;
	case DL_MIFARE_ULTRALIGHT_C:
		strcpy(s, "DL_MIFARE_ULTRALIGHT_C");
		break;
	case DL_NTAG_203:
		strcpy(s, "DL_NTAG_203");
		break;
	case DL_NTAG_210:
		strcpy(s, "DL_NTAG_210");
		break;
	case DL_NTAG_212:
		strcpy(s, "DL_NTAG_212");
		break;
	case DL_NTAG_213:
		strcpy(s, "DL_NTAG_213");
		break;
	case DL_NTAG_215:
		strcpy(s, "DL_NTAG_215");
		break;
	case DL_NTAG_216:
		strcpy(s, "DL_NTAG_216");
		break;
	case DL_MIKRON_MIK640D:
		strcpy(s, "DL_MIKRON_MIK640D");
		break;
	case DL_MIFARE_MINI:
		strcpy(s, "DL_MIFARE_MINI");
		break;
	case DL_MIFARE_CLASSIC_1K:
		strcpy(s, "DL_MIFARE_CLASSIC_1K");
		break;
	case DL_MIFARE_CLASSIC_4K:
		strcpy(s, "DL_MIFARE_CLASSIC_4K");
		break;
	case DL_MIFARE_PLUS_S_2K:
		strcpy(s, "DL_MIFARE_PLUS_S_2K");
		break;
	case DL_MIFARE_PLUS_S_4K:
		strcpy(s, "DL_MIFARE_PLUS_S_4K");
		break;
	case DL_MIFARE_PLUS_X_2K:
		strcpy(s, "DL_MIFARE_PLUS_X_2K");
		break;
	case DL_MIFARE_PLUS_X_4K:
		strcpy(s, "DL_MIFARE_PLUS_X_4K");
		break;
	case DL_MIFARE_DESFIRE:
		strcpy(s, "DL_MIFARE_DESFIRE");
		break;
	case DL_MIFARE_DESFIRE_EV1_2K:
		strcpy(s, "DL_MIFARE_DESFIRE_EV1_2K");
		break;
	case DL_MIFARE_DESFIRE_EV1_4K:
		strcpy(s, "DL_MIFARE_DESFIRE_EV1_4K");
		break;
	case DL_MIFARE_DESFIRE_EV1_8K:
		strcpy(s, "DL_MIFARE_DESFIRE_EV1_8K");
		break;
	case DL_IMEI_UID:
		strcpy(s, "DL_IMEI_UID");
		break;
	default:
		strcpy(s, "UNSUPPORTED CARD");
	}

	return s;
}
//------------------------------------------------------------------------------
void Printout(void) {

	printf("Set:\n"
			"     [0] for 0\n");
	printf(
			"     [1] for 1\n");
	print_ln('-');
	printf("Enter your choice:\n");
}
//------------------------------------------------------------------------------
void SetMode(void) {
	fflush(stdin);
}
//------------------------------------------------------------------------------
void print_ln_len(char symbol, uint8_t cnt) {

	for (int i = 0; i < cnt; i++)
		printf("%c", symbol);

	printf("\n");
}
//------------------------------------------------------------------------------
void print_ln(char symbol) {

	print_ln_len(symbol, DEFAULT_LINE_LEN);
}
//------------------------------------------------------------------------------
void print_hex(const uint8_t *data, uint32_t len, const char *delimiter) {

	for (int i = 0; i < len; i++) {
		printf("%02X", data[i]);
		if ((delimiter != NULL) && (i < (len - 1)))
			printf("%c", *delimiter);
	}
}
//------------------------------------------------------------------------------
void print_hex_ln(const uint8_t *data, uint32_t len, const char *delimiter) {

	print_hex(data, len, delimiter);
	printf("\n");
}
//==============================================================================
